#!/usr/bin/perl

# usage: sync.pl
#		(csv file path)
#		(template file path)
#		(mediawiki URL)
#		(mediawiki user name)
#		(mediawiki password)
#		[pagename field no, default = 1]
#		[section field no, default = 0 (empty)]
#		[id field no, default = 0 (auto detect)]
#		[index table file path]
#		[http user name, default = &apos;&apos; (empty)]
#		[http password, default = &apos;&apos; (empty)]
#		[&apos;force&apos; for update all words without diff check]

# @param csv name of ID-fixed csv will be @param{csv}.sync.csv

use DictoolCSV;
use WikiSecFormatter;
use WikiPage;
use WordInfoList;
use MediaWikiUA;
use WikiLinkPages;
use IndexTable;
use CategorySet;

################################################################

if($#ARGV &lt; 1) {
  showUsage();
  die;
}

($csvFilepath,
 $tmplFilepath,
 $mediaWikiRootUrl,
 $mediaWikiUsername,
 $mediaWikiPass,
 $pagenameCol,
 $sectionCol,
 $idCol,
 $indexTableFilepath,
 $httpUsername,
 $httpPass,
 $force
) = (@ARGV);

$pagenameCol--;
$sectionCol--;
$idCol--;

$httpAuth = new AuthInfo($httpUsername, $httpPass);
$wikiAuth = new AuthInfo($mediaWikiUsername, $mediaWikiPass);

$wordInfoList = new WordInfoList($mediaWikiRootUrl, $httpAuth);
$wordInfoList-&gt;load();

$csv = new DictoolCSV($wordInfoList, $csvFilepath,
		      $pagenameCol, $sectionCol, $idCol, $normalIdCol);

print &quot;reading csv.\n&quot;;
$csv-&gt;load();

# simply write to mediawiki

$ua = new MediaWikiUA($mediaWikiRootUrl, $httpAuth);
print &quot;login...\n&quot;;
$ua-&gt;login($wikiAuth);

print &quot;authorized.\n&quot;;

$updateTime = new Timestamp($ua);

$indexTable = new IndexTable($indexTableFilepath);
$categorySet = new CategorySet();

$tmplStr = &quot;&quot;;
open(TMPL, $tmplFilepath) || die &quot;template file read failed.&quot;;
while(<tmpl>) {
  $tmplStr .= $_; # FIXME: which encoding is used for tmpl file?
}
close(TMPL);

$formatter = new WikiSecFormatter($tmplStr, $csv, $indexTable, $categorySet);

$force = ($force eq &quot;force&quot;);

my %ids;
foreach my $id (@{$csv-&gt;{wordIds}}) {

  update($id, $force);

}

# disable deleted sections
foreach my $id ($wordInfoList-&gt;getDeletedIds(%ids)) {

  my $pagename = $wordInfoList-&gt;getPagename($id); # don&apos;t use csv-&gt;getPagename because this id is not shown in CSV.
  my $page = new WikiPage($pagename);

  print &quot;deleting old data of &quot;.$id.&quot;\n&quot;;

  $page-&gt;load($ua);

  $page-&gt;disableSectionAGNode($id);
  $wordInfoList-&gt;deleteWordInfo($id);

  $page-&gt;save($ua);
}

################

$csv-&gt;save();
$wordInfoList-&gt;save($wikiAuth);

$indexTable-&gt;save($ua);
$categorySet-&gt;save($ua);
$updateTime-&gt;save();

print &quot;done.\n&quot;;

exit 0;

################################################################
# sub routines

sub update {

  my ($id, $force) = @_;

  $ids{$id} = 1;

  my @wordInfo = $csv-&gt;getWordInfo($id);

  if((!$force) &amp;&amp; (!$wordInfoList-&gt;checkUpdateNeeded($id, @wordInfo))) {
    return;
  }

  my $oldPagename = $wordInfoList-&gt;getPagename($id);
  my $pagename = $csv-&gt;getPagename($id);
  my $parentId = $csv-&gt;getNormId($id);

  # update parent node if exists
  if(defined($parentId) &amp;&amp; $parentId != $id) {
    update($parentId, 1);
  }

  #
  $wordInfoList-&gt;setWordInfo($id, $pagename, $parentId, @wordInfo);

  my $page = new WikiPage($pagename);
  $page-&gt;load($ua);

  my $oldContent;

  if(defined($oldPagename)) {

    if($oldPagename ne $pagename) {

      # move word to another page

      my $oldPage = new WikiPage($oldPagename);
      $oldPage-&gt;load($ua);
      $oldContent = $oldPage-&gt;getSectionContent($id);

      $oldPage-&gt;disableSectionAGNode($id, $pagename);
      $oldPage-&gt;setContent($id, &apos;&apos;);
      $oldPage-&gt;save($ua);

      my $linkInfo = new WikiLinkPages($oldPagename);
      my $linksRef = $linkInfo-&gt;getLinksRef($ua);

      foreach my $linkPagename (@$linksRef) {

	my $linkPage = new WikiPage($linkPagename);
	$linkPage-&gt;load($ua);
	$linkPage-&gt;redirectLinks($id, $oldPagename, $pagename);
	$linkPage-&gt;save($ua);
      }

    } else {

      $oldContent = $page-&gt;getSectionContent($id);
    }
  }

  print &quot;writing to &quot;.Encode::encode(&apos;euc-jp&apos;, $pagename).&quot;\n&quot;;

  my $agStr = $formatter-&gt;format($id, $sectionCol, $updateTime);

  $page-&gt;setSectionAGNode($id, $agStr);

  if(defined($oldContent)) {
    $page-&gt;setContent($id, $oldContent);
  }

  $page-&gt;save($ua);

}

sub showUsage {

  print &lt;<eof; dicsync="" ver.="" 1.07="" (20070404)="" (c)="" 2006-2007,="" national="" institute="" of="" informatics,="" japan.="" usage:="" dicsync.pl="" (csv)="" (template="" file="" path)="" (mediawiki="" url="" (